
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace skybox_keyboard
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        
        
        Matrix view, proj, world;
        Vector3 position,forward,up,sideways;
        Effect effect;
        float angle = 0.0f;
        SkyBox skybox;
        TextureCube texture;
        SpriteFont font;
        SpriteBatch sb;
        Model teapot;



        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            position = new Vector3(0, 0, 200);
            up = new Vector3(0, 1, 0);
            forward = new Vector3(0, 0, -1);
            sideways = Vector3.Cross(forward, up);


            proj = Matrix.CreatePerspectiveFieldOfView(
                MathHelper.ToRadians(90),
                (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height,
                0.01f, 1000.0f);


           
            skybox = new SkyBox("snow", graphics.GraphicsDevice,Content);
            sb = new SpriteBatch(graphics.GraphicsDevice) ;
            base.Initialize();
        }

        private void InitializeTransform()
        {
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadContent()
        {
 
            effect = Content.Load<Effect>("env_mapping");
            teapot = Content.Load<Model>("teapot");
            texture = Content.Load<TextureCube>("snow");
            font = Content.Load<SpriteFont>("SpriteFont1");
 


            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadContent()
        {


                Content.Unload();
            
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {


            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            KeyboardState kb=Keyboard.GetState();

            if(kb.IsKeyDown(Keys.Escape))
                this.Exit();

            // TODO: Add your update logic here
            angle += 0.01f;
            world = Matrix.CreateRotationX(angle) * Matrix.CreateRotationY(angle * 1.5f);
             
            sideways = Vector3.Cross(forward,up );
            
            if(kb.IsKeyDown(Keys.LeftShift)){ // rotate if shift is held
                if (kb.IsKeyDown(Keys.Up)) // tilt up
                {                        
                    Matrix r=Matrix.CreateFromAxisAngle(sideways, (float)Math.PI / 100.0f);
                    forward = Vector3.Transform(forward, r);
                    up = Vector3.Transform(up, r);
                }
                if (kb.IsKeyDown(Keys.Down))
                { // tilt down
                    Matrix r = Matrix.CreateFromAxisAngle(sideways, -(float)Math.PI / 100.0f);
                    forward = Vector3.Transform(forward, r);
                    up = Vector3.Transform(up, r);
                }

               

                if (kb.IsKeyDown(Keys.Right))
                { // bank right
                    Matrix r = Matrix.CreateFromAxisAngle(forward, (float)Math.PI / 100.0f);
                    up = Vector3.Transform(up, r);
                }
                if (kb.IsKeyDown(Keys.Left))
                { // bank left
                    Matrix r = Matrix.CreateFromAxisAngle(forward, -(float)Math.PI / 100.0f);
                    up = Vector3.Transform(up, r);

                }


            }
            else{
                if (kb.IsKeyDown(Keys.Up))
                { // forward
                    position += forward;
                }
                if (kb.IsKeyDown(Keys.Down))
                { // backward
                    position -= forward;
                }

               

                if (kb.IsKeyDown(Keys.Right))
                { // turn right
                    Matrix r = Matrix.CreateFromAxisAngle(up, -(float)Math.PI / 100.0f);
                    forward = Vector3.Transform(forward, r);
                }
                if (kb.IsKeyDown(Keys.Left))
                { // turn left
                    Matrix r = Matrix.CreateFromAxisAngle(up, (float)Math.PI / 100.0f);
                    forward = Vector3.Transform(forward, r);
                }
            }

            view = Matrix.CreateLookAt(
             position,  // place camera
             position + forward, // point it forward
             up
             );

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;

            // TODO: Add your drawing code here

            skybox.Draw(graphics.GraphicsDevice, position, view * proj);
            
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;
            graphics.GraphicsDevice.RenderState.DepthBufferEnable = true; ;
            graphics.GraphicsDevice.RenderState.DepthBufferWriteEnable = true; ;

            effect.CurrentTechnique = effect.Techniques["EnvReflect"];
            effect.Parameters["view_proj_matrix"].SetValue(world * view * proj);
            effect.Parameters["Light1_Position"].SetValue(new Vector4(0, 300, 0, 1));
            effect.Parameters["view_position"].SetValue(position);
            effect.Parameters["Light_Ambient"].SetValue(new Vector4(0.1f, 0.1f, 0.1f, 1));
            effect.Parameters["inv_world_matrix"].SetValue(Matrix.Invert(world));
            effect.Parameters["world_matrix"].SetValue(world);
            effect.Parameters["Texture"].SetValue(texture);


            
            // need to set the environment effect of each part of the mesh
            foreach (ModelMesh m in teapot.Meshes)
            {
                
                foreach (ModelMeshPart mmp in m.MeshParts)
                {
                    mmp.Effect = effect;
                }
                
                m.Draw();
            }

           

            sb.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.BackToFront, SaveStateMode.SaveState);

            // Draw Hello World
            string output = "Use <Shift>up/down/left/right to move around";

            // Find the center of the string
            Vector2 FontOrigin = font.MeasureString(output) / 2;
            // Draw the string
            sb.DrawString(font, output, FontOrigin, Color.LightGreen,
                0, FontOrigin, 1.0f, SpriteEffects.None, 0.5f);

            sb.End();


            base.Draw(gameTime);
        }
    }
}